Package: keylist
Title: Lightweight List Extensions that Enforce Unique Keys
Version: 1.0.0
Authors@R: 
    person("Luke", "Jenkins", , "luke-jenkins-dev@outlook.com", role = c("aut", "cre", "cph"), comment = c(ORCID = "0000-0002-7206-7242"))
Description: Provides two lightweight keylist S3 classes 'klist' and 'knlist': extensions of list that enforce unique keys, supporting either mixed named/unnamed elements or fully named elements, ensuring predictable key-value access.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
URL: https://lj-jenkins.github.io/keylist/,
        https://github.com/LJ-Jenkins/keylist
BugReports: https://github.com/LJ-Jenkins/keylist/issues
NeedsCompilation: yes
Packaged: 2026-04-23 10:06:53 UTC; lukej
Author: Luke Jenkins [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-7206-7242>)
Maintainer: Luke Jenkins <luke-jenkins-dev@outlook.com>
Repository: CRAN
Date/Publication: 2026-04-27 14:20:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-01 17:13:22 UTC; unix
Archs: keylist.so.dSYM
