# Generated by roxygen2: do not edit by hand

S3method("[",klist)
S3method("[",knlist)
S3method("[<-",knlist)
S3method("[[<-",knlist)
S3method("names<-",klist)
S3method("names<-",knlist)
S3method(as.array,klist)
S3method(as.array,knlist)
S3method(as.data.frame,klist)
S3method(as.data.frame,knlist)
S3method(as.keylist,default)
S3method(as.klist,default)
S3method(as.klist,list)
S3method(as.knlist,default)
S3method(as.knlist,list)
S3method(as.list,klist)
S3method(as.list,knlist)
S3method(as.matrix,klist)
S3method(as.matrix,knlist)
S3method(as.vector,klist)
S3method(as.vector,knlist)
S3method(c,klist)
S3method(c,knlist)
S3method(print,klist)
S3method(print,knlist)
export(as.keylist)
export(as.klist)
export(as.knlist)
export(is.keylist)
export(is.klist)
export(is.knlist)
export(keylist)
export(keylist.append)
export(klist)
export(knlist)
useDynLib(keylist, .registration = TRUE)
